module Pod
  module Downloader
    require 'cocoapods-downloader/gem_version'
    require 'cocoapods-downloader/api'
    require 'cocoapods-downloader/api_exposable'
    require 'cocoapods-downloader/base'

    autoload :Bazaar,      'cocoapods-downloader/bazaar'
    autoload :Git,         'cocoapods-downloader/git'
    autoload :Http,        'cocoapods-downloader/http'
    autoload :Mercurial,   'cocoapods-downloader/mercurial'
    autoload :Subversion,  'cocoapods-downloader/subversion'

    # Denotes the error generated by a Downloader
    #
    class DownloaderError < StandardError; end

    # @return [Hash{Symbol=>Class}] The concrete classes of the supported
    #         strategies by key.
    #
    def self.downloader_class_by_key
      {
        :bzr  => Bazaar,
        :git  => Git,
        :hg   => Mercurial,
        :http => Http,
        :svn  => Subversion,
      }
    end

    # Identifies the concrete strategy for the given options.
    #
    # @param  [Hash{Symbol}] options
    #         The options for which a strategy is needed.
    #
    # @return [Symbol] The symbol associated with a concrete strategy.
    # @return [Nil] If no suitable concrete strategy could be selected.
    #
    def self.strategy_from_options(options)
      common = downloader_class_by_key.keys & options.keys
      if common.count == 1
        common.first
      end
    end

    # @return [Downloader::Base] A concrete downloader according to the
    #         options.
    #
    def self.for_target(target_path, options)
      options = Hash[options.map { |k, v| [k.to_sym, v] }]

      if target_path.nil?
        raise DownloaderError, 'No target path provided.'
      end

      if options.nil? || options.empty?
        raise DownloaderError, 'No source URL provided.'
      end

      strategy = strategy_from_options(options)
      unless strategy
        raise DownloaderError, 'Unsupported download strategy ' \
          "`#{options.inspect}`."
      end

      url = options[strategy]
      sub_options = options.dup
      sub_options.delete(strategy)
      klass = downloader_class_by_key[strategy]
      klass.new(target_path, url, sub_options)
    end
  end
end
